@@ -5,12 +5,12 @@
 #include "application.h"
 
 #include "buildcfg.h"
+#include "config.h"
 #include "font.h"
 #include "gallery.h"
 #include "imagelist.h"
 #include "info.h"
 #include "loader.h"
-#include "str.h"
 #include "sway.h"
 #include "ui.h"
 #include "viewer.h"
@@ -24,7 +24,6 @@
 #include <stdlib.h>
 #include <string.h>
 #include <sys/wait.h>
-#include <time.h>
 #include <unistd.h>
 
 // Special ids for windows size and position
@@ -48,9 +47,9 @@ struct watchfd {
 };
 
 /* Application event queue (list). */
-struct event_entry {
+struct event_queue {
+    struct list list;
     struct event event;
-    struct event_entry* next;
 };
 
 /** Application context */
@@ -60,7 +59,7 @@ struct application {
     struct watchfd* wfds; ///< FD polling descriptors
     size_t wfds_num;      ///< Number of polling FD
 
-    struct event_entry* events;  ///< Event queue
+    struct event_queue* events;  ///< Event queue
     pthread_mutex_t events_lock; ///< Event queue lock
     int event_signal;            ///< Queue change notification
 
@@ -68,7 +67,7 @@ struct application {
     struct action_seq sigusr2; ///< Actions applied by USR2 signal
 
     event_handler ehandler; ///< Event handler for the current mode
-    struct rect window;     ///< Preferable window position and size
+    struct wndrect window;  ///< Preferable window position and size
     char* app_id;           ///< Application id (app_id name)
 };
 
@@ -80,7 +79,7 @@ static struct application ctx;
  */
 static void sway_setup(void)
 {
-    struct rect parent;
+    struct wndrect parent;
     bool fullscreen;
     bool absolute = false;
     int ipc;
@@ -111,21 +110,8 @@ static void sway_setup(void)
         ctx.window.y = parent.y;
     }
 
-    if (!ctx.app_id) {
-        // create unique application id
-        struct timespec ts;
-        if (clock_gettime(CLOCK_MONOTONIC, &ts) == 0) {
-            char app_id[64];
-            const uint64_t uid = ((uint64_t)ts.tv_sec << 32) | ts.tv_nsec;
-            snprintf(app_id, sizeof(app_id), APP_NAME "_%" PRIx64, uid);
-            str_dup(app_id, &ctx.app_id);
-        } else {
-            str_dup(APP_NAME, &ctx.app_id);
-        }
-    }
-
     // set window position via sway rules
-    sway_add_rules(ipc, ctx.app_id, ctx.window.x, ctx.window.y, absolute);
+    sway_add_rules(ipc, ctx.window.x, ctx.window.y, absolute);
 
     sway_disconnect(ipc);
 }
@@ -136,15 +122,17 @@ static void handle_event_queue(__attribute__((unused)) void* data)
     notification_reset(ctx.event_signal);
 
     while (ctx.events && ctx.state == loop_run) {
-        struct event_entry* entry;
+        struct event_queue* entry = NULL;
         pthread_mutex_lock(&ctx.events_lock);
-        entry = ctx.events;
         if (ctx.events) {
-            ctx.events = ctx.events->next;
+            entry = ctx.events;
+            ctx.events = list_remove(entry);
         }
         pthread_mutex_unlock(&ctx.events_lock);
-        ctx.ehandler(&entry->event);
-        free(entry);
+        if (entry) {
+            ctx.ehandler(&entry->event);
+            free(entry);
+        }
     }
 }
 
@@ -154,7 +142,7 @@ static void handle_event_queue(__attribute__((unused)) void* data)
  */
 static void append_event(const struct event* event)
 {
-    struct event_entry* entry;
+    struct event_queue* entry;
 
     // create new entry
     entry = malloc(sizeof(*entry));
@@ -162,19 +150,10 @@ static void append_event(const struct event* event)
         return;
     }
     memcpy(&entry->event, event, sizeof(entry->event));
-    entry->next = NULL;
 
     // add to queue tail
     pthread_mutex_lock(&ctx.events_lock);
-    if (ctx.events) {
-        struct event_entry* last = ctx.events;
-        while (last->next) {
-            last = last->next;
-        }
-        last->next = entry;
-    } else {
-        ctx.events = entry;
-    }
+    ctx.events = list_append(ctx.events, entry);
     pthread_mutex_unlock(&ctx.events_lock);
 
     notification_raise(ctx.event_signal);
@@ -376,6 +355,7 @@ void app_create(void)
     ctx.window.width = SIZE_FROM_PARENT;
     ctx.window.height = SIZE_FROM_PARENT;
     ctx.ehandler = viewer_handle;
+    str_dup(APP_NAME, &ctx.app_id);
 
     action_create("reload", &ctx.sigusr1);
     action_create("next_file", &ctx.sigusr2);
@@ -407,13 +387,11 @@ void app_destroy(void)
     }
     free(ctx.wfds);
 
-    while (ctx.events) {
-        struct event_entry* entry = ctx.events;
-        ctx.events = ctx.events->next;
-        if (entry->event.type == event_load) {
-            image_free(entry->event.param.load.image);
+    list_for_each(ctx.events, struct event_queue, it) {
+        if (it->event.type == event_load) {
+            image_free(it->event.param.load.image);
         }
-        free(entry);
+        free(it);
     }
     if (ctx.event_signal != -1) {
         notification_free(ctx.event_signal);
@@ -473,15 +451,12 @@ bool app_init(const char** sources, size_t num)
         ctx.window.height = pm->height;
     }
 
-    if (!ctx.app_id) {
-        str_dup(APP_NAME, &ctx.app_id);
-    }
-
+    // connect to wayland
     if (!ui_init(ctx.app_id, ctx.window.width, ctx.window.height)) {
         return false;
     }
 
-    // event queue notification
+    // create event queue notification
     ctx.event_signal = notification_create();
     if (ctx.event_signal != -1) {
         app_watch(ctx.event_signal, handle_event_queue, NULL);
@@ -624,24 +599,21 @@ void app_redraw(void)
     const struct event event = {
         .type = event_redraw,
     };
-    struct event_entry* prev = NULL;
-    struct event_entry* it = ctx.events;
 
-    // remove the same event to append the new one to tail
-    while (it) {
-        struct event_entry* next = it->next;
+    // remove the same event to append new one to tail
+    pthread_mutex_lock(&ctx.events_lock);
+    list_for_each(ctx.events, struct event_queue, it) {
         if (it->event.type == event_redraw) {
-            if (prev) {
-                prev->next = next;
-            } else {
-                ctx.events = next;
+            if (list_is_last(it)) {
+                pthread_mutex_unlock(&ctx.events_lock);
+                return;
             }
+            ctx.events = list_remove(it);
             free(it);
             break;
         }
-        prev = it;
-        it = next;
     }
+    pthread_mutex_unlock(&ctx.events_lock);
 
     append_event(&event);
 }
@@ -677,17 +649,18 @@ void app_on_drag(int dx, int dy)
     const struct event event = { .type = event_drag,
                                  .param.drag.dx = dx,
                                  .param.drag.dy = dy };
-    struct event_entry* it = ctx.events;
 
     // merge with existing event
-    while (it) {
+    pthread_mutex_lock(&ctx.events_lock);
+    list_for_each(ctx.events, struct event_queue, it) {
         if (it->event.type == event_drag) {
             it->event.param.drag.dx += dx;
             it->event.param.drag.dy += dy;
+            pthread_mutex_unlock(&ctx.events_lock);
             return;
         }
-        it = it->next;
     }
+    pthread_mutex_unlock(&ctx.events_lock);
 
     append_event(&event);
 }
