@@ -51,26 +51,40 @@ void image_rotate(struct image* ctx, size_t angle)
     }
 }
 
-void image_thumbnail(struct image* image, size_t size, bool antialias)
+void image_thumbnail(struct image* image, size_t size, bool fill,
+                     bool antialias)
 {
     struct pixmap thumb;
     struct image_frame* frame;
     const struct pixmap* full = &image->frames[0].pm;
-
     const float scale_width = 1.0 / ((float)full->width / size);
     const float scale_height = 1.0 / ((float)full->height / size);
-    const float scale = min(scale_width, scale_height);
-    const size_t thumb_width = scale * full->width;
-    const size_t thumb_height = scale * full->height;
+    const float scale =
+        fill ? max(scale_width, scale_height) : min(scale_width, scale_height);
+    size_t thumb_width = scale * full->width;
+    size_t thumb_height = scale * full->height;
+    ssize_t offset_x, offset_y;
+
+    if (fill) {
+        offset_x = size / 2 - thumb_width / 2;
+        offset_y = size / 2 - thumb_height / 2;
+        thumb_width = size;
+        thumb_height = size;
+    } else {
+        offset_x = 0;
+        offset_y = 0;
+    }
 
     // create thumbnail
     if (!pixmap_create(&thumb, thumb_width, thumb_height)) {
         return;
     }
     if (antialias) {
-        pixmap_scale_bicubic(full, &thumb, 0, 0, scale, image->alpha);
+        pixmap_scale_bicubic(full, &thumb, offset_x, offset_y, scale,
+                             image->alpha);
     } else {
-        pixmap_scale_nearest(full, &thumb, 0, 0, scale, image->alpha);
+        pixmap_scale_nearest(full, &thumb, offset_x, offset_y, scale,
+                             image->alpha);
     }
 
     image_free_frames(image);
