@@ -10,7 +10,6 @@
 #include "imagelist.h"
 #include "keybind.h"
 #include "loader.h"
-#include "str.h"
 #include "ui.h"
 
 #include <stdarg.h>
@@ -20,8 +19,6 @@
 #include <sys/timerfd.h>
 #include <unistd.h>
 
-// clang-format off
-
 /** Display modes. */
 enum info_mode {
     mode_viewer,
@@ -35,6 +32,7 @@ static const char* mode_names[] = {
 };
 #define MODES_NUM 2
 
+// clang-format off
 /** Field names. */
 static const char* field_names[] = {
     [info_file_name] = "name",
@@ -49,6 +47,7 @@ static const char* field_names[] = {
     [info_status] = "status",
 };
 #define INFO_FIELDS_NUM ARRAY_SIZE(field_names)
+// clang-format on
 
 /** Positions of text info block. */
 enum block_position {
@@ -100,8 +99,6 @@ static const struct field_scheme default_gallery_br[] = {
     { .type = info_status,    .title = false },
 };
 
-// clang-format on
-
 // Space between text layout and window edge
 #define TEXT_PADDING 10
 
@@ -577,7 +574,6 @@ void info_switch(const char* mode)
 
 void info_switch_help(void)
 {
-
     if (ctx.help) {
         for (size_t i = 0; i < ctx.help_num; i++) {
             free(ctx.help[i].data);
@@ -586,33 +582,27 @@ void info_switch_help(void)
         ctx.help = NULL;
         ctx.help_num = 0;
     } else {
-        const struct keybind* kb = keybind_get();
-        size_t num = 0;
-
         // get number of bindings
-        while (kb) {
-            if (kb->help) {
+        size_t num = 0;
+        list_for_each(keybind_get(), struct keybind, it) {
+            if (it->help) {
                 ++num;
             }
-            kb = kb->next;
         }
         if (num == 0) {
             return;
         }
 
-        // create help layer
+        // create help layer in reverse order
         ctx.help = calloc(1, num * sizeof(*ctx.help));
         if (!ctx.help) {
             return;
         }
         ctx.help_num = num;
-        // fill keybinds help (they are stored in reverse order)
-        kb = keybind_get();
-        while (kb) {
-            if (kb->help) {
-                font_render(kb->help, &ctx.help[--num]);
+        list_for_each(keybind_get(), struct keybind, it) {
+            if (it->help) {
+                font_render(it->help, &ctx.help[--num]);
             }
-            kb = kb->next;
         }
     }
 }
