@@ -12,7 +12,6 @@
 #include "info.h"
 #include "keybind.h"
 #include "loader.h"
-#include "str.h"
 #include "ui.h"
 
 #include <stdio.h>
@@ -359,10 +358,12 @@ static void reset_state(void)
  */
 static bool skip_image(void)
 {
-    size_t index = image_list_skip(fetcher_current()->index);
+    size_t index;
+    const size_t current = fetcher_current()->index;
 
+    index = image_list_skip(current);
     while (index != IMGLIST_INVALID && !fetcher_open(index)) {
-        index = image_list_next_file(index);
+        index = image_list_skip(index);
     }
 
     return (index != IMGLIST_INVALID);
